import 'dart:async';
import 'dart:developer';
import 'dart:io';

import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:phoenix_widgets/phoenix_widgets.dart';
import 'package:webview_flutter/webview_flutter.dart';

import 'custom_widget/custom_app_bar.dart';
import 'custom_widget/custom_progress_bar.dart';

class PhoenixWebview extends StatefulWidget {
  const PhoenixWebview({
    Key? key,
    required this.title,
    required this.url,
    this.isForRegistration = false,
    this.isShowRefresh,
    this.userAgent,
    this.bottomNavigationBar,
    this.refreshIcons,
    this.customJavaScripts,
    this.navigationDelegate,
    this.jsToRun,
    this.noInternetHandler,
  }) : super(key: key);
  final String title, url;
  final bool isForRegistration;
  final bool? isShowRefresh;
  final String? userAgent, customJavaScripts;
  final Widget? bottomNavigationBar, refreshIcons;
  final Function? navigationDelegate;
  final List<String>? jsToRun;
  final Widget? noInternetHandler;

  @override
  State<PhoenixWebview> createState() => _PhoenixWebviewState();
}

class _PhoenixWebviewState extends State<PhoenixWebview> {
  bool _isLoading = true;
  double percentage = 0;

  late WebViewController _myController; //to handle JS

  bool isOnline = true;
  final Connectivity connectivity = Connectivity();
  late StreamSubscription<ConnectivityResult> connectivitySubscription;

  Future<void> updateConnectionStatus(ConnectivityResult result) async {
    final connectivityResult = await connectivity.checkConnectivity();
    isOnline = connectivityResult != ConnectivityResult.none;

    if (isOnline) {
      _reload();
    }
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initConnectivity() async {
    late ConnectivityResult result;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      result = await connectivity.checkConnectivity();
    } on PlatformException catch (e) {
      log('Could not check connectivity status', error: e);
      return;
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) {
      return Future.value(null);
    }

    return updateConnectionStatus(result);
  }

  @override
  void initState() {
    super.initState();

    initConnectivity();

    // Check connectivity
    connectivitySubscription =
        connectivity.onConnectivityChanged.listen(updateConnectionStatus);

    if (Platform.isAndroid) WebView.platform = SurfaceAndroidWebView();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      Timer(const Duration(seconds: 10), () {
        if (mounted) {
          setState(() {
            _isLoading = false;
          });
        }
      });
    });
  }

  @override
  void dispose() {
    connectivitySubscription.cancel();
    super.dispose();
  }

  Future<bool> _onBackPressed() async {
    if (isOnline) {
      bool canNavigate = await _myController.canGoBack();
      if (canNavigate) {
        _myController.goBack();
        return Future.value(false);
      } else {
        await _myController.clearCache();
        Navigator.of(context).pop();
        return Future.value(true);
      }
    } else {
      Navigator.of(context).pop();
      return Future.value(false);
    }
  }

  void _reload() async {
    setState(() {
      percentage = 0;
      _isLoading = true;
    });
    await _myController.reload();
  }

  @override
  Widget build(BuildContext context) {
    final _isDoneLoading = !_isLoading && percentage == 1.0;

    return Scaffold(
      appBar: CustomAppBar(
        centerTitle: true,
        title: PhoenixTitle(title: widget.title),
        leading: PhoenixBackButton(
          callback: _onBackPressed,
        ),
        actions: (widget.isShowRefresh ?? false)
            ? [
                IconButton(
                  onPressed: _reload,
                  icon: widget.refreshIcons ?? const Icon(Icons.refresh),
                ),
                const SizedBox(
                  width: 2,
                )
              ]
            : [],
        bottom: ProgressBar(
          isDoneLoading: _isDoneLoading,
          percentage: percentage,
        ),
      ),
      body: WillPopScope(
        onWillPop: _onBackPressed,
        child: !isOnline
            ? widget.noInternetHandler ??
                const Center(
                  child: Icon(Icons.signal_wifi_connected_no_internet_4),
                )
            : Stack(
                children: [
                  Positioned.fill(
                    child: WebView(
                      initialUrl: widget.url,
                      javascriptMode: JavascriptMode.unrestricted,
                      onProgress: (progress) {
                        setState(() {
                          percentage = progress / 100;
                          _isLoading = progress == 0;
                        });
                        if (progress == 1.0) {
                          Future.delayed(const Duration(seconds: 1), () {
                            setState(() {
                              _isLoading = false;
                            });
                          });
                        }
                      },
                      onWebViewCreated: (controller) {
                        _myController = controller;
                      },
                      navigationDelegate: (NavigationRequest request) async {
                        if (widget.navigationDelegate != null) {
                          widget.navigationDelegate;
                        }
                        return NavigationDecision.navigate;
                      },
                      userAgent: widget.userAgent ?? '',
                      onPageFinished: (str) {
                        Future.delayed(const Duration(milliseconds: 2000), () {
                          setState(() {
                            _isLoading = false;
                            readJs();
                          });
                        });
                      },
                    ),
                  ),
                  _isLoading
                      ? const Positioned.fill(
                          child: Center(
                            child: SizedBox(
                              width: 50.0,
                              height: 50.0,
                              child: CircularProgressIndicator(
                                strokeWidth: 1.5,
                                valueColor:
                                    AlwaysStoppedAnimation(Color(0xFF67C1BF)),
                              ),
                            ),
                          ),
                        )
                      : Container()
                ],
              ),
      ),
      bottomNavigationBar: widget.bottomNavigationBar,
    );
  }

  void readJs() async {
    if (widget.isForRegistration && widget.jsToRun!.isNotEmpty) {
      for (int i = 0; i < widget.jsToRun!.length; i++) {
        await _myController.runJavascript(widget.jsToRun![i]);
      }

      if (widget.customJavaScripts != null) {
        await _myController.runJavascript(widget.customJavaScripts!);
      }
    }
  }
}
